@@ -24,7 +24,7 @@ import os
 import re
 import json
 import numpy as np
-import pkuseg
+import spacy_pkuseg
 import traceback
 
 from jionlp import logging
@@ -41,6 +41,7 @@ DIR_PATH = os.path.dirname(os.path.abspath(__file__))
 class Chunking(object):
     """ 根据词性和规则，从文本中找出名词类短语
     """
+
     def __init__(self):
         pass
 
@@ -85,9 +86,10 @@ class ChineseKeyPhrasesExtractor(object):
         #  '朝方转交普京', '最高司令官金正恩']
 
     """
+
     def __init__(self, ):
         self.unk_topic_prominence_value = 0.
-        
+
     def _prepare(self):
         # 词性预处理
         # 词性参考 https://github.com/lancopku/pkuseg-python/blob/master/tags.txt
@@ -95,19 +97,22 @@ class ChineseKeyPhrasesExtractor(object):
         self.pos_name = set(pkuseg_postag_loader().keys())
         self.pos_exception = set(['u', 'p', 'c', 'y', 'e', 'o', 'w'])
         self.loose_pos_name = self.pos_name - self.pos_exception
-        self.strict_pos_name = ['a', 'n', 'j', 'nr', 'ns', 'nt', 'nx', 'nz', 
+        self.strict_pos_name = ['a', 'n', 'j', 'nr', 'ns', 'nt', 'nx', 'nz',
                                 'ad', 'an', 'vn', 'vd', 'vx']
 
         # 去除冗余短语的规则
-        self.redundant_strict_pattern = re.compile('[\*\|`\;:丨－\<\>]')  # 有一个字符即抛弃
-        self.redundant_loose_pattern = re.compile('[/\d\.\-:=a-z+,%]+')  # 全部是该字符即抛弃
+        self.redundant_strict_pattern = re.compile(
+            '[\*\|`\;:丨－\<\>]')  # 有一个字符即抛弃
+        self.redundant_loose_pattern = re.compile(
+            '[/\d\.\-:=a-z+,%]+')  # 全部是该字符即抛弃
         self.extra_date_ptn = re.compile('\d{1,2}[月|日]')
-        
+
         # 加载 idf，计算其 oov 均值
         self.idf_dict = idf_loader()
-        self.median_idf = sorted(self.idf_dict.values())[len(self.idf_dict) // 2]
-        self.seg = pkuseg.pkuseg(postag=True)  # 北大分词器
-        
+        self.median_idf = sorted(self.idf_dict.values())[
+            len(self.idf_dict) // 2]
+        self.seg = spacy_pkuseg.pkuseg(postag=True)  # 北大分词器
+
         # 短语长度权重字典，调整绝大多数的短语要位于 2~6 个词之间
         # 根据人工抽取的关键短语结果，短语词长有一个泊松分布，而根据 idf 和 lda 概率的结果，也有一个
         # 分布，两者之间存在偏差，因此，直接对短语长度分布采用权重进行调节，使抽取分布逼近人工的分布。
@@ -115,18 +120,18 @@ class ChineseKeyPhrasesExtractor(object):
             1: 1, 2: 5.6, 3: 1.1, 4: 2.0, 5: 0.7, 6: 0.9, 7: 0.48,
             8: 0.43, 9: 0.24, 10: 0.15, 11: 0.07, 12: 0.05}
         self.phrases_length_control_none = 0.01  # 在大于 7 时选取
-        
+
         # 短语词性组合权重字典
-        with open(os.path.join(DIR_PATH, 'pos_combine_weights.json'), 
+        with open(os.path.join(DIR_PATH, 'pos_combine_weights.json'),
                   'r', encoding='utf8') as f:
             self.pos_combine_weights_dict = json.load(f)
-        
+
         # 读取停用词文件
         self.stop_words = stopwords_loader()
-        
+
         # 加载 lda 模型参数
         self._lda_prob_matrix()
-        
+
     def _lda_prob_matrix(self):
         """ 读取 lda 模型有关概率分布文件，并计算 unk 词的概率分布 """
         # 读取 p(topic|word) 概率分布文件，由于 lda 模型过大，不方便加载并计算
@@ -134,23 +139,24 @@ class ChineseKeyPhrasesExtractor(object):
         # 但是，由于默认的 lda 模型 topic_num == 100，事实上，lda 模型是否在
         # 预测的文档上收敛对结果影响不大（topic_num 越多，越不影响）。
 
-        dict_dir_path = os.path.join(os.path.dirname(os.path.dirname(DIR_PATH)), 'dictionary')
+        dict_dir_path = os.path.join(os.path.dirname(
+            os.path.dirname(DIR_PATH)), 'dictionary')
 
         with open(os.path.join(dict_dir_path, 'topic_word_weight.json'),
                   'r', encoding='utf8') as f:
             self.topic_word_weight = json.load(f)
         self.word_num = len(self.topic_word_weight)
-        
+
         # 读取 p(word|topic) 概率分布文件
         with open(os.path.join(dict_dir_path, 'word_topic_weight.json'),
                   'r', encoding='utf8') as f:
             self.word_topic_weight = json.load(f)
         self.topic_num = len(self.word_topic_weight)
-        
+
         self._topic_prominence()  # 预计算主题突出度
-    
+
     def __call__(self, text, top_k=5, with_weight=False,
-                 func_word_num=1, stop_word_num=0, 
+                 func_word_num=1, stop_word_num=0,
                  max_phrase_len=25,
                  topic_theta=0.5, allow_pos_weight=True,
                  strict_pos=True, allow_length_weight=True,
@@ -164,7 +170,7 @@ class ChineseKeyPhrasesExtractor(object):
             # 初始化加载
             if self.unk_topic_prominence_value == 0.:
                 self._prepare()
-                
+
             # 配置参数
             if without_location_name:
                 if 'ns' in self.strict_pos_name:
@@ -193,7 +199,7 @@ class ChineseKeyPhrasesExtractor(object):
 
             # step1: 分句，使用北大的分词器 pkuseg 做分词和词性标注
             sentences_list = split_sentence(text, criterion='fine')
-            
+
             sentences_segs_list = list()
             counter_segs_list = list()
             for sen in sentences_list:
@@ -246,15 +252,16 @@ class ChineseKeyPhrasesExtractor(object):
                         candidate_phrase = sen_segs[i: i + n]
 
                         # 由于 pkuseg 的缺陷，日期被识别为 n 而非 t，故删除日期
-                        res = self.extra_date_ptn.match(candidate_phrase[-1][0])
+                        res = self.extra_date_ptn.match(
+                            candidate_phrase[-1][0])
                         if res is not None:
                             continue
 
                         # 找短语过程中需要进行过滤，分为严格、宽松规则
-                        if not strict_pos:  
+                        if not strict_pos:
                             rule_flag = self._loose_candidate_phrases_rules(
                                 candidate_phrase, func_word_num=func_word_num,
-                                max_phrase_len=max_phrase_len,  
+                                max_phrase_len=max_phrase_len,
                                 stop_word_num=stop_word_num)
                         else:
                             rule_flag = self._strict_candidate_phrases_rules(
@@ -265,18 +272,20 @@ class ChineseKeyPhrasesExtractor(object):
                         # 由于 pkuseg 的缺陷，会把一些杂质符号识别为 n、v、adj，故须删除
                         redundant_flag = False
                         for item in candidate_phrase:
-                            matched = self.redundant_strict_pattern.search(item[0])
+                            matched = self.redundant_strict_pattern.search(
+                                item[0])
                             if matched is not None:
                                 redundant_flag = True
                                 break
-                            matched = self.redundant_loose_pattern.search(item[0])
+                            matched = self.redundant_loose_pattern.search(
+                                item[0])
 
                             if matched is not None and matched.group() == item[0]:
                                 redundant_flag = True
                                 break
                         if redundant_flag:
                             continue
-                            
+
                         # 如果短语中包含了某些不想要的词，则跳过
                         if remove_words_list is not None:
                             unwanted_phrase_flag = False
@@ -303,15 +312,17 @@ class ChineseKeyPhrasesExtractor(object):
                             if len(candidate_phrase) == 1:
                                 start_end_pos = candidate_phrase[0][1]
                             elif len(candidate_phrase) >= 2:
-                                start_end_pos = candidate_phrase[0][1] + '|' + candidate_phrase[-1][1]
-                            pos_weight = self.pos_combine_weights_dict.get(start_end_pos, 1.0)
+                                start_end_pos = candidate_phrase[0][1] + \
+                                    '|' + candidate_phrase[-1][1]
+                            pos_weight = self.pos_combine_weights_dict.get(
+                                start_end_pos, 1.0)
                         else:
                             pos_weight = 1.0
 
                         # 条件七：短语的权重需要乘上 '长度权重'
                         if allow_length_weight:
                             length_weight = self.phrases_length_control_dict.get(
-                                len(sen_segs_weights[i: i + n]), 
+                                len(sen_segs_weights[i: i + n]),
                                 self.phrases_length_control_none)
                         else:
                             length_weight = 1.0
@@ -326,23 +337,25 @@ class ChineseKeyPhrasesExtractor(object):
                         else:
                             topic_weight = 0.0
 
-                        candidate_phrase_weight = sum(sen_segs_weights[i: i + n])
+                        candidate_phrase_weight = sum(
+                            sen_segs_weights[i: i + n])
                         candidate_phrase_weight *= length_weight * pos_weight
                         candidate_phrase_weight += topic_weight * topic_theta
 
-                        candidate_phrase_string = ''.join([tup[0] for tup in candidate_phrase])
+                        candidate_phrase_string = ''.join(
+                            [tup[0] for tup in candidate_phrase])
                         if remove_phrases_list is not None:
                             if candidate_phrase_string in remove_phrases_list:
                                 continue
                         if candidate_phrase_string not in candidate_phrases_dict:
                             candidate_phrases_dict.update(
-                                {candidate_phrase_string: [candidate_phrase, 
+                                {candidate_phrase_string: [candidate_phrase,
                                                            candidate_phrase_weight]})
 
             # step5: 将 overlapping 过量的短语进行去重过滤
             # 尝试了依据权重高低，将较短的短语替代重复了的较长的短语，但效果不好，故删去
             candidate_phrases_list = sorted(
-                candidate_phrases_dict.items(), 
+                candidate_phrases_dict.items(),
                 key=lambda item: len(item[1][0]), reverse=True)
 
             de_duplication_candidate_phrases_list = list()
@@ -355,7 +368,7 @@ class ChineseKeyPhrasesExtractor(object):
 
             # step6: 按重要程度进行排序，选取 top_k 个
             candidate_phrases_list = sorted(
-                de_duplication_candidate_phrases_list, 
+                de_duplication_candidate_phrases_list,
                 key=lambda item: item[1][1], reverse=True)
 
             if with_weight:
@@ -375,7 +388,8 @@ class ChineseKeyPhrasesExtractor(object):
             return final_res
 
         except Exception as e:
-            logging.error('the text is illegal. \n{}'.format(traceback.format_exc()))
+            logging.error('the text is illegal. \n{}'.format(
+                traceback.format_exc()))
             return list()
 
     @staticmethod
@@ -384,16 +398,16 @@ class ChineseKeyPhrasesExtractor(object):
         """ 计算 mmr 相似度，用于考察信息量 """
         sim_ratio = 0.0
         candidate_info = set([item[0] for item in candidate_item[1][0]])
-        
+
         for de_du_item in de_duplication_candidate_phrases_list:
             no_info = set([item[0] for item in de_du_item[1][0]])
             common_part = candidate_info & no_info
             if sim_ratio < len(common_part) / len(candidate_info):
                 sim_ratio = len(common_part) / len(candidate_info)
         return sim_ratio
-        
+
     def _loose_candidate_phrases_rules(self, candidate_phrase,
-                                       max_phrase_len=25, 
+                                       max_phrase_len=25,
                                        func_word_num=1, stop_word_num=0):
         """ 按照宽松规则筛选候选短语，对词性和停用词宽松 """
         # 条件一：一个短语不能超过 12个 token
@@ -420,7 +434,7 @@ class ChineseKeyPhrasesExtractor(object):
         if candidate_phrase[len(candidate_phrase)-1][1] in ['v', 'd']:
             return False
         if candidate_phrase[0][0] in self.stop_words:
-            return False 
+            return False
         if candidate_phrase[len(candidate_phrase)-1][0] in self.stop_words:
             return False
 
@@ -432,8 +446,8 @@ class ChineseKeyPhrasesExtractor(object):
         if has_stop_words_count > stop_word_num:
             return False
         return True
-    
-    def _strict_candidate_phrases_rules(self, candidate_phrase, 
+
+    def _strict_candidate_phrases_rules(self, candidate_phrase,
                                         max_phrase_len=25):
         """ 按照严格规则筛选候选短语，严格限制在名词短语 """
         # 条件一：一个短语不能超过 12个 token
@@ -460,37 +474,44 @@ class ChineseKeyPhrasesExtractor(object):
         #    if item[0] in self.stop_words and item[1] not in self.strict_pos_name:
         #        return False
         return True
-        
+
     def _topic_prominence(self):
         """ 计算每个词语的主题突出度，并保存在内存 """
-        init_prob_distribution = np.array([self.topic_num for i in range(self.topic_num)])
-        
+        init_prob_distribution = np.array(
+            [self.topic_num for i in range(self.topic_num)])
+
         topic_prominence_dict = dict()
         for word in self.topic_word_weight:
             conditional_prob_list = list()
             for i in range(self.topic_num):
                 if str(i) in self.topic_word_weight[word]:
-                    conditional_prob_list.append(self.topic_word_weight[word][str(i)])
+                    conditional_prob_list.append(
+                        self.topic_word_weight[word][str(i)])
                 else:
                     conditional_prob_list.append(1e-5)
             conditional_prob = np.array(conditional_prob_list)
-            
-            tmp_dot_log_res = np.log2(np.multiply(conditional_prob, init_prob_distribution))
-            kl_div_sum = np.dot(conditional_prob, tmp_dot_log_res)  # kl divergence
+
+            tmp_dot_log_res = np.log2(np.multiply(
+                conditional_prob, init_prob_distribution))
+            kl_div_sum = np.dot(
+                conditional_prob, tmp_dot_log_res)  # kl divergence
             topic_prominence_dict.update({word: float(kl_div_sum)})
-            
+
         tmp_list = [i[1] for i in tuple(topic_prominence_dict.items())]
         max_prominence = max(tmp_list)
         min_prominence = min(tmp_list)
         for k, v in topic_prominence_dict.items():
-            topic_prominence_dict[k] = (v - min_prominence) / (max_prominence - min_prominence)
-            
+            topic_prominence_dict[k] = (
+                v - min_prominence) / (max_prominence - min_prominence)
+
         self.topic_prominence_dict = topic_prominence_dict
-        
+
         # 计算未知词汇的主题突出度，由于停用词已经预先过滤，所以这里不需要再考停用词无突出度
-        tmp_prominence_list = [item[1] for item in self.topic_prominence_dict.items()]
-        self.unk_topic_prominence_value = sum(tmp_prominence_list) / (2 * len(tmp_prominence_list))
-        
+        tmp_prominence_list = [item[1]
+                               for item in self.topic_prominence_dict.items()]
+        self.unk_topic_prominence_value = sum(
+            tmp_prominence_list) / (2 * len(tmp_prominence_list))
+
 
 if __name__ == '__main__':
     title = '巴黎圣母院大火：保安查验火警失误 现场找到7根烟头'
@@ -515,12 +536,12 @@ if __name__ == '__main__':
            '此外，目前尚难以判定究竟是短路引发大火还是火灾造成短路。25日，即巴黎圣母院发生震惊全球的严重火灾10天后，' \
            '法国司法警察刑事鉴定专家进入失火现场展开勘查取证工作，标志着火因调查的技术程序正式启动。' \
            '此前，由于灾后建筑结构仍不稳定和现场积水过多，调查人员一直没有真正开始采集取样。'
-    
+
     ckpe_obj = ChineseKeyPhrasesExtractor()
     key_phrases = ckpe_obj(text, topic_theta=1)
     print('key_phrases_1topic: ', key_phrases)
     key_phrases = ckpe_obj(text, topic_theta=0)
     print('key_phrases_notopic: ', key_phrases)
-    key_phrases = ckpe_obj(text, allow_length_weight=False, topic_theta=0.5, max_phrase_len=8)
+    key_phrases = ckpe_obj(text, allow_length_weight=False,
+                           topic_theta=0.5, max_phrase_len=8)
     print('key_phrases_05topic: ', key_phrases)
-
