@@ -35,7 +35,7 @@
 import copy
 import collections
 
-import pkuseg
+import spacy_pkuseg
 
 from jionlp.dictionary.dictionary_loader import china_location_loader
 from jionlp.dictionary.dictionary_loader import world_location_loader
@@ -108,7 +108,7 @@ class LocationRecognizer(object):
         self.pkuseg = None
         
     def _prepare(self):
-        self.pkuseg = pkuseg.pkuseg(postag=True)
+        self.pkuseg = spacy_pkuseg.pkuseg(postag=True)
         
         china_loc = china_location_loader()
         world_loc = world_location_loader()
